@@ -26,9 +26,31 @@ class. This Kivy app class instance can be directly accessed from the main app c
 :attr:`~ae.gui_app.MainAppBase.framework_app` attribute.
 
 
+kivy app class events
+^^^^^^^^^^^^^^^^^^^^^
+
+This portion is adding to the :ref:`application events` provided by :class:`~ae.gui_app.MainAppBase` the following
+events redirected from the Kivy :class:`~kivy.app.App` class (the original Kivy event/callback-method name is
+given in brackets):
+
+* on_kivy_app_build (build).
+* on_kivy_app_start (on_start)
+* on_app_pause (on_pause)
+* on_app_resume (on_resume)
+* on_kivy_app_stop (on_stop)
+
+
 kivy widget classes
 -------------------
 
+The widgets provided by this portion are based on the kivy widgets and are respecting the :ref:`app-state-variables`
+specifying the desired app style (dark or light) and font size.
+
+Most of them also change automatically the :ref:`application-flow`.
+
+The following widgets provided by this portion will be registered in the kivy widget class maps by importing this module
+for to be available for your app:
+
 * :class:`AppStateSlider`: :class:`~kivy.uix.slider.Slider>` changing the value of :ref:`app-state-variables`.
 * :class:`FlowButton`: :class:`ImageButton` for to change the application flow.
 * :class:`FlowDropDown`: :class:`~kivy.uix.dropdown.DropDown` for to process application flow.
@@ -42,35 +64,6 @@ kivy widget classes
 * :class:`OptionalButton`: dynamic kivy widget based on :class:`FlowButton` which can be dynamically hidden.
 
 
-text input widget
-^^^^^^^^^^^^^^^^^
-
-Until version 0.1.43 of this portion the background and text color switched with the light_theme app state.
-Now all colors left unchanged (before only the ones with <unchanged>):
-
-* background_color: Window.clearcolor            # default: 1, 1, 1, 1
-* cursor_color: app.font_color                   # default: 1, 0, 0, 1
-* disabled_foreground_color: <unchanged>         # default: 0, 0, 0, .5
-* foreground_color: app.font_color               # default: 0, 0, 0, 1
-* hint_text_color: <unchanged>                   # default: 0.5, 0.5, 0.5, 1.0
-* selection_color: <unchanged>                   # default: 0.1843, 0.6549, 0.8313, .5
-
-For to implement a nice dark background for the dark theme we would need also to change the images in the properties:
-background_active, background_disabled_normal and self.background_normal.
-
-TODO: replace/extend TextInputCutCopyPaste bubble widget of kivy.uix.textinput.TextInput with (1) theme colors and
-(2) additional buttons for to add(Ctrl+Insert)/delete(Ctrl+Delete) autocompletion text entry (for mobile platforms).
-Very ugly, but when FlowBubble gets implemented (inherited from kivy.uix.bubble.Bubble) then we could patch
-kivy.uix.textinput.TextInputCutCopyPaste like::
-
-    class TextInputCutCopyPaste(FlowBubble):
-        ...
-
-    from kivy.uix import textinput
-    textinput.TextInputCutCopyPaste = TextInputCutCopyPaste
-    from kivy.uix.textinput import TextInput
-
-
 unit tests
 ----------
 
@@ -101,10 +94,13 @@ from kivy.metrics import sp
 from kivy.properties import (                                                               # type: ignore
     BooleanProperty, DictProperty, ListProperty, ObjectProperty, StringProperty)
 from kivy.uix.behaviors import ButtonBehavior, ToggleButtonBehavior                         # type: ignore
+from kivy.uix.bubble import BubbleButton                                                    # type: ignore
 from kivy.uix.dropdown import DropDown                                                      # type: ignore
 from kivy.uix.popup import Popup                                                            # type: ignore
 from kivy.uix.slider import Slider                                                          # type: ignore
-from kivy.uix.textinput import TextInput                                                    # type: ignore
+import kivy.uix.textinput                                                                   # type: ignore
+# noinspection PyProtectedMember
+from kivy.uix.textinput import TextInput, TextInputCutCopyPaste as _TextInputCutCopyPaste   # type: ignore
 from kivy.uix.widget import Widget                                                          # type: ignore
 
 from ae.base import sys_platform                                                            # type: ignore
@@ -126,7 +122,7 @@ from ae.kivy_help import HelpBehavior, HelpLayout, HelpToggler
 from ae.kivy_relief_canvas import ReliefCanvas                                              # type: ignore
 
 
-__version__ = '0.1.48'
+__version__ = '0.1.54'
 
 
 kivy.require('2.0.0')
@@ -327,7 +323,7 @@ def ensure_tap_kwargs_refs(init_kwargs: Dict[str, Any], tap_widget: Widget):
     :param init_kwargs:         kwargs of the widgets __init__ method.
     :param tap_widget:          reference to the tap widget.
 
-    This alternative version is only 10 % faster but much more confusing than the current implementation::
+    This alternative version is only 10 % faster but much less clean than the current implementation::
 
         if 'tap_kwargs' not in init_kwargs:
             init_kwargs['tap_kwargs'] = dict()
@@ -498,10 +494,75 @@ class FlowDropDown(ContainerChildrenAutoWidthBehavior, DynamicChildrenBehavior,
         return super().on_touch_down(touch)
 
 
+class ExtTextInputCutCopyPaste(_TextInputCutCopyPaste):                                     # pragma: no cover
+    """ overwrite kivy.uix.textinput.TextInputCutCopyPaste for to translate options and add autocompletion options. """
+    def __init__(self, **kwargs):
+        """ reset monkey patch of kivy.uix.textinput.TextInputCutCopyPaste done in FlowInput._show_cut_copy_paste().
+
+        reset has to be done before super() call below, for to prevent endless recursion because else the other
+        super(cls, instance) call (in python2 style within TextInputCutCopyPaste.__init__()) results
+        in the same instance (instead of the overwritten instance).
+        """
+        kivy.uix.textinput.TextInputCutCopyPaste = _TextInputCutCopyPaste
+        super().__init__(**kwargs)
+
+    def on_parent(self, instance: Widget, value: Widget):
+        """ overwritten for to translate BubbleButton texts and for to add extra menus for to add/delete ac texts.
+
+        :param instance:        self.
+        :param value:           kivy main window.
+        """
+        super().on_parent(instance, value)
+        textinput = self.textinput
+        if not textinput:
+            return
+
+        cont = self.content
+        font_size = App.get_running_app().main_app.font_size
+
+        for child in cont.children:
+            child.font_size = font_size
+            child.text = get_txt(child.text)
+
+        if not textinput.readonly:
+            # memorize/forget complete text to/from autocomplete because dropdown is not visible if this bubble is
+            self.add_widget(BubbleButton(text=get_txt("Memorize"), font_size=font_size,
+                                         on_release=textinput.extend_ac_with_text))
+            self.add_widget(BubbleButton(text=get_txt("Forget"), font_size=font_size,
+                                         on_release=textinput.delete_text_from_ac))
+
+        # estimate container size (exact calc not possible because button width / texture_size[0] is still 100 / 0)
+        width = cont.padding[0] + cont.padding[2] + len(cont.children) * (cont.spacing[0] + sp(126))
+        height = cont.padding[1] + cont.padding[3] + font_size * 1.5
+        self.size = width, height
+
+
 class FlowInput(HelpBehavior, TextInput):                                                             # pragma: no cover
-    """ text input/edit widget with optional autocompletion. """
+    """ text input/edit widget with optional autocompletion.
+
+    Until version 0.1.43 of this portion the background and text color of :class:`FlowInput` did automatically
+    get switched by a change of the light_theme app state. Now all colors left unchanged (before only the ones
+    with <unchanged>):
+
+    * background_color: Window.clearcolor            # default: 1, 1, 1, 1
+    * cursor_color: app.font_color                   # default: 1, 0, 0, 1
+    * disabled_foreground_color: <unchanged>         # default: 0, 0, 0, .5
+    * foreground_color: app.font_color               # default: 0, 0, 0, 1
+    * hint_text_color: <unchanged>                   # default: 0.5, 0.5, 0.5, 1.0
+    * selection_color: <unchanged>                   # default: 0.1843, 0.6549, 0.8313, .5
+
+    For to implement a dark background for the dark theme we would need also to change the images in the properties:
+    background_active, background_disabled_normal and self.background_normal.
+
+    Also the images/colors of the bubble that is showing e.g. on long press of the TextInput widget (cut/copy/paste/...)
+    kept unchanged - only the font_size get adapted and the bubble button texts get translated. For that the class
+    :class:`ExtTextInputCutCopyPaste` provided by this portion inherits from the original bubble class
+    :class:`~kivy.uix.textinput.TextInputCutCopyPaste`. Additionally the original bubble class gets monkey patched
+    shortly/temporarily in the moment of the instantiation for to translate the bubble menu options, change the font
+    sizes and add additional menu options for to memorize/forget auto-completion texts.
+    """
     focus_flow_id = StringProperty()        #: flow id that will be set when this widget get focus
-    unfocus_flow_id = StringProperty()      #: flow id that will be set when this widget get focus
+    unfocus_flow_id = StringProperty()      #: flow id that will be set when this widget lost focus
 
     auto_complete_texts: List[str] = ListProperty()     #: list of autocompletion texts
     auto_complete_selector_index_ink: Tuple[float, float, float, float] = ListProperty((0.69, 0.69, 0.69, 1))
@@ -535,13 +596,29 @@ class FlowInput(HelpBehavior, TextInput):
         chi[self._matching_ac_index].square_fill_ink = self.auto_complete_selector_index_ink
         self.suggestion_text = self._matching_ac_texts[self._matching_ac_index][len(self.text):]    # type: ignore #mypy
 
-    def _delete_ac_text(self):
-        ac_text = self._matching_ac_texts[self._matching_ac_index]
-        self.auto_complete_texts.remove(ac_text)
-        self.on_text(self, self.text)       # redraw autocompletion dropdown
+    def _delete_ac_text(self, ac_text: str = ""):
+        if not ac_text and self._matching_ac_texts:
+            ac_text = self._matching_ac_texts[self._matching_ac_index]
+        if ac_text in self.auto_complete_texts:
+            self.auto_complete_texts.remove(ac_text)
+            self.on_text(self, self.text)       # type: ignore  # redraw autocompletion dropdown
 
-    def _extend_ac_texts(self):
-        self.auto_complete_texts.insert(0, self.text)
+    def delete_text_from_ac(self, *_args):
+        """ check if current text is in autocompletion list and if yes then remove it.
+
+        called by FlowInput kbd event handler and from menu button added by ExtTextInputCutCopyPaste.on_parent().
+
+        :param _args:           unused event args.
+        """
+        self._delete_ac_text(self.text)
+
+    def extend_ac_with_text(self, *_args):
+        """ add non-empty text to autocompletion texts.
+
+        :param _args:           unused event args.
+        """
+        if self.text:
+            self.auto_complete_texts.insert(0, self.text)
 
     def keyboard_on_key_down(self, window: Any, keycode: Tuple[int, str], text: str, modifiers: List[str]) -> bool:
         """ overwritten TextInput/FocusBehavior kbd event handler.
@@ -568,7 +645,7 @@ class FlowInput(HelpBehavior, TextInput):
                 self._delete_ac_text()
 
         if key_name == 'insert' and 'ctrl' in modifiers:
-            self._extend_ac_texts()
+            self.extend_ac_with_text()
 
         return super().keyboard_on_key_down(window, keycode, text, modifiers)
 
@@ -615,6 +692,11 @@ class FlowInput(HelpBehavior, TextInput):
         self.text = selector.text
         self._ac_dropdown.dismiss()
 
+    def _show_cut_copy_paste(self, *args, **kwargs):
+        kivy.uix.textinput.TextInputCutCopyPaste = ExtTextInputCutCopyPaste  # reset in ExtTextInputCutCopyPaste.__init_
+        super()._show_cut_copy_paste(*args, **kwargs)
+        kivy.uix.textinput.TextInputCutCopyPaste = _TextInputCutCopyPaste    # reset here too if already instantiated
+
 
 class FlowPopup(ContainerChildrenAutoWidthBehavior, DynamicChildrenBehavior, ReliefCanvas, Popup):    # pragma: no cover
     """ pop up widget used for dialogs and other top-most or modal windows. """
@@ -694,6 +776,7 @@ class FrameworkApp(App):
 
         :return:                root widget (Main instance) of this app.
         """
+        self.main_app.call_method('on_kivy_app_build')
         Window.bind(on_resize=self.win_pos_size_change,
                     left=self.win_pos_size_change,
                     top=self.win_pos_size_change,
@@ -917,6 +1000,16 @@ class KivyMainApp(HelpAppBase):
 
     # overwritten and helper methods
 
+    def call_method_delayed(self, delay: float, method: str, *args, **kwargs):
+        """ in `delay` seconds call method with the passed args, catching and logging exceptions preventing app exit.
+
+        :param delay:       delay in seconds when to call the method specified by the `method` argument.
+        :param method:      name of the main app method to call.
+        :param args:        args passed to the main app method to be called.
+        :param kwargs:      kwargs passed to the main app method to be called.
+        """
+        Clock.schedule_once(lambda dt: self.call_method(method, *args, **kwargs), timeout=delay)
+
     def change_light_theme(self, light_theme: bool):
         """ change font and window clear/background colors to match 'light'/'black' themes.
 
@@ -951,12 +1044,7 @@ class KivyMainApp(HelpAppBase):
         help_vars = dict()
         hlw = None
         if activate:
-            target = activator
-            if self.flow_id:
-                help_id = self.help_flow_id(self.flow_id)
-                target = self.help_widget(help_id, help_vars)
-                if target is activator:
-                    help_id = ''
+            target, help_id = self.help_target_and_id(help_vars)
             hlw = HelpLayout(target=target,
                              ps_hints=layout_ps_hints(*target.to_window(*target.pos), *target.size,
                                                       self.framework_win.width, self.framework_win.height))
@@ -997,10 +1085,11 @@ class KivyMainApp(HelpAppBase):
         get_txt.switch_lang(self.lang_code)
         self.change_light_theme(self.light_theme)
 
-        win_rect = self.win_rectangle
-        if win_rect:                                # is empty tuple at very first app start
-            Window.left, Window.top = win_rect[:2]
-            Window.size = win_rect[2:]
+        if sys_platform() not in ('android', 'ios'):    # ignore last win pos on android/iOS, use always the full screen
+            win_rect = self.win_rectangle
+            if win_rect:                                # is empty tuple at very first app start
+                Window.left, Window.top = win_rect[:2]
+                Window.size = win_rect[2:]
 
     def on_flow_widget_focused(self):
         """ set focus to the widget referenced by the current flow id. """
@@ -1148,9 +1237,6 @@ class KivyMainApp(HelpAppBase):
                 container.clear_widgets()                       # clear container for Popup only
             popup_instance = popup_class(**popup_kwargs)        # new instance if kbd covering popup
 
-        if not hasattr(popup_instance, 'close') and hasattr(popup_instance, 'dismiss'):
-            popup_instance.close = popup_instance.dismiss       # create close() method alias for DropDown.dismiss()
-
         popup_instance.open(parent)
 
         return popup_instance
